/****************** ks_imp_includes.h ******************************/
/*
*  Include files for Kogut-Susskind dynamical improved action application
*/

/* Include files */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <complex.h>
#include <su3.h>
#include LATDEF    /* global variables for lattice fields,
		defined in Make_template so that we can compile in
		different directories */
#include <macros.h>
#include <comdefs.h>
#include <io_lat.h>
#include <generic_ks.h>
#include <generic.h>

/* site pointer array for nn and 3n make_NN_gathers
  (nearest and 3rd nearest neighbors)
  If not a single workstation, also must sort send and receive lists.
  NOTE THIS GETS OUR CLAWS INTO THINGS THAT ARE USUALLY HIDDEN IN THE
  COM_*.c FILES, AND IS NOT GUARANTEED PORTABLE TO ALL MACHINES */
extern  int ** neighbor;
#ifndef VANILLA
extern  comlink **neighborlist, **neighborlist_send;
#endif

#ifdef FN
#define dslash dslash_fn
#endif
#ifdef EO
#define dslash dslash_eo
#endif

#define SSE
#undef SSE
#ifdef SSE
#define mult_su3_mat_vec(args...) sse_mult_su3_mat_vec(## args)
#define mult_su3_mat_vec_sum_4dir(args...) sse_mult_su3_mat_vec_sum_4dir(## args)
#define mult_adj_su3_mat_vec(args...) sse_mult_adj_su3_mat_vec(## args)
#define mult_adj_su3_mat_vec_4dir(args...) sse_mult_adj_su3_mat_vec_4dir(## args)
#define mult_su3_mat_hwvec(args...) sse_mult_su3_mat_hwvec(## args)
#define mult_adj_su3_mat_hwvec(args...) sse_mult_adj_su3_mat_hwvec(## args)
#define mult_su3_nn(args...) sse_mult_su3_nn(## args)
#define mult_su3_na(args...) sse_mult_su3_na(## args)
#define su3_projector(args...) sse_su3_projector(## args)
#define scalar_mult_add_su3_matrix(args...) sse_scalar_mult_add_su3_matrix(## args)
#endif

#define USE_PREFETCH
#ifdef USE_PREFETCH
#   define PREFETCH_T0(  addr ) \
        __asm__( ".byte 0x0f,0x18,0x08" ::"a"(addr) )
#   define PREFETCH_T1(  addr ) \
        __asm__( ".byte 0x0f,0x18,0x10" ::"a"(addr) )
#   define PREFETCH_NTA( addr ) \
        __asm__( ".byte 0x0f,0x18,0x00" ::"a"(addr) )
#   define PREFETCH( addr ) \
        __asm__( ".byte 0x0f,0x18,0x00" ::"a"(addr) )
#else
#   define PREFETCH_T0( addr )
#   define PREFETCH_T1( addr )
#   define PREFETCH_NTA( addr )
#   define PREFETCH( addr )
#endif

/* prototypes for functions in high level code */
int setup();
int update();
void make_path_table();
void update_h( float eps );
void update_u( float eps );
void gauge_force( float eps );
void grsource_imp( field_offset dest, float mass, int parity );
void imp_gauge_force( float eps );
void eo_fermion_force( float eps, int nflavors, field_offset x_off );
void eo_fermion_force_3f( float eps, int nflav1, field_offset x1_off,
	int nflav2, field_offset x2_off  );
double imp_gauge_action( );
double hmom_action( );
double fermion_action( );
void ranmom();


int spectrum_old( float mass );
int nl_spectrum( float mass );
int spectrum_mom( float qmass, float amass );
int spectrum_imp( float qmass, float amass );
int spectrum_nlpi2( float qmass, float amass );
int pion_rot();
void hvy_pot( field_offset links );
void f_measure( field_offset phi_off, field_offset xxx_off, float mass );
void g_measure( void );
void make_loop_table();
void gauge_field_copy(field_offset src,field_offset dest);
void load_longlinks();
void load_fatlinks();
void clear_latvec(field_offset v,int parity);
void copy_latvec(field_offset src,field_offset dest,int parity);
void scalar_mult_add_latvec(field_offset src1,field_offset src2,
			    float scalar,field_offset dest,int parity);
void scalar2_mult_add_su3_vector(su3_vector *a, float s1, su3_vector *b, 
				 float s2, su3_vector *c);
void scalar2_mult_add_latvec(field_offset src1,float scalar1,
			     field_offset src2,float scalar2,
			     field_offset dest,int parity);
void scalar_mult_latvec(field_offset src,float scalar,
			field_offset dest,int parity);

void path_product( int *dir, int length);
int ks_congrad( field_offset src, field_offset dest, float mass,
     int niter, float rsqmin, int parity, float *rsq );
void dslash_fn( field_offset src, field_offset dest, int parity );
void dslash_fn_special( field_offset src, field_offset dest,
    int parity, msg_tag **tag, int start );
void dslash_eo( field_offset src, field_offset dest, int parity );
void dslash_eo_special( field_offset src, field_offset dest,
    int parity, msg_tag **tag, int start );
int mat_invert_uml(field_offset src, field_offset dest, float mass );
int mat_invert_cg( field_offset src, field_offset dest, float mass );
void check_invert( field_offset src, field_offset dest, float mass );
void checkmul_imp( field_offset src, float mass );

void rephase( int flag );
EXTERN gauge_file *startlat_p;
