;
; sse_su3_projector( su3_vector *a, su3_vector *b, su3_matrix *c)
;
; C = a (outer) b* 

global sse_su3_projector
sse_su3_projector:
	push		ebp
	mov		ebp,esp
	push		eax
	push		ebx
	push		ecx
	mov		eax,[ebp+8]			; su3_vector *a
	mov		ebx,[ebp+12]			; su3_vector *b
	mov		ecx,[ebp+16]			; su3_matrix *c

	;  bring in B vector (d,e,f)
	movups		xmm0,[ebx]			; ei,er,di,dr
	movaps		xmm1,xmm0
	shufps		xmm0,xmm0,0xA0			; er,er,dr,dr
	shufps		xmm1,xmm1,0xF5			; ei,ei,di,di
	movups		xmm2,[ebx+8]			; fi,fr,ei,er
	movaps		xmm3,xmm2
	shufps		xmm2,xmm2,0xA0			; fr,fr,er,er
	shufps		xmm3,xmm3,0xF5			; fi,fi,di,di
	movlhps		xmm4,xmm0
	movhlps		xmm4,xmm2			; dr,dr,fr,fr
	movlhps		xmm5,xmm1
	movhlps		xmm5,xmm3			; di,di,fi,fi
	; xmm0: er, er, dr, dr
	; xmm1: ei, ei, di, di
	; xmm2: fr, fr, er, er
	; xmm3: fi, fi, ei, ei
	; xmm4: dr, dr, fr, fr
	; xmm5: di, di, fi, fi

        ; *******************************************************************   
        ; *******************************************************************   
	; bring in A vector, first element (a,b,c)
	movhps		xmm6,[eax]			; ai,ar,x,x
	shufps		xmm6,xmm6,0xEE			; ai,ar,ai,ar
	movaps		xmm7,xmm6
	shufps		xmm7,xmm7,0x11			; ar,ai,ar,ai
	; xmm6: ai, ar, ai, ar
	; xmm7: ar, ai, ar, ai

	mulps		xmm6,xmm0
	mulps		xmm7,xmm1
	; xmm6: ai.er, ar.er, ai.dr, ar.dr
	; xmm7: ar.ei, ai.ei, ar.di, ai.di

	xorps		xmm7,[negate]			; -ar.ei, ai.ei, -ar.di, ai.di
	addps		xmm7,xmm6
	movups		[ecx],xmm7

        ; *******************************************************************   
        ; *******************************************************************   
	; bring in A vector, first and second elements (a,b,c)
	movups		xmm6,[eax]			; bi,br,ai,ar
	movaps		xmm7,xmm6
	shufps		xmm7,xmm7,0xB1			; br,bi,ar,ai
	; xmm6: bi, br, ai, ar
	; xmm7: br, bi, ar, ai

	mulps		xmm6,xmm4
	mulps		xmm7,xmm5
	; xmm6: bi.dr, br.dr, ai.fr, ar.fr
	; xmm7: br.di, bi.di, ar.fi, ai.fi

	xorps		xmm7,[negate]			; -br.di, bi.di, -ar.fi, ai.fi
	addps		xmm7,xmm6
	movups		[ecx+16],xmm7

        ; *******************************************************************   
        ; *******************************************************************   
	; bring in A vector, second element (a,b,c)
	movhps		xmm6,[eax+8]			; bi,br,x,x
	shufps		xmm6,xmm6,0xEE			; bi,br,bi,br
	movaps		xmm7,xmm6
	shufps		xmm7,xmm7,0x11			; br,bi,br,bi
	; xmm6: bi, br, bi, br
	; xmm7: br, bi, br, bi

	mulps		xmm6,xmm2
	mulps		xmm7,xmm3
	; xmm6: bi.fr, br.fr, bi.er, br.er
	; xmm7: br.fi, bi.fi, br.ei, bi.ei
	
	xorps		xmm7,[negate]			; -br.fi, bi.fi, -br.ei, bi.ei
	addps		xmm7,xmm6
	movups		[ecx+32],xmm7

        ; *******************************************************************   
        ; *******************************************************************   
	; bring in A vector, third element (a,b,c)
	movhps		xmm6,[eax+16]			; ci,cr,x,x
	shufps		xmm6,xmm6,0xEE			; ci,cr,ci,cr
	movaps		xmm7,xmm6
	shufps		xmm7,xmm7,0x11			; cr,ci,cr,ci
	movaps		xmm4,xmm6
	movaps		xmm5,xmm7
	; xmm6: ci, cr, ci, cr
	; xmm7: cr, ci, cr, ci
	; xmm4: ci, cr, ci, cr
	; xmm5: cr, ci, cr, ci

	mulps		xmm6,xmm0
	mulps		xmm7,xmm1
	; xmm6: ci.er, cr.er, ci.dr, cr.dr
	; xmm7: cr.ei, ci.ei, cr.di, ci.di

	xorps		xmm7,[negate]			; -cr.ei, ci.ei, -cr.di, ci.di
	addps		xmm7,xmm6
	movups		[ecx+48],xmm7
	
        ; *******************************************************************   
        ; *******************************************************************   
	; and do the last element, c.f*
	mulps		xmm4,xmm2
	mulps		xmm5,xmm3
	; xmm4: ci.fr, cr.fr, ci.er, cr.er
	; xmm5: cr.fi, ci.fi, cr.ei, ci.ei

	xorps		xmm5,[negate]			; -cr.ei, ci.ei, -cr.di, ci.di
	addps		xmm5,xmm4
	movhps		[ecx+64],xmm5

here:	pop	ecx
	pop	ebx
	pop	eax
	mov	esp,ebp
	pop	ebp
	ret
	
	align		16
negate:	dd		0x00000000
	dd		0x80000000
	dd		0x00000000
	dd		0x80000000
	
