/* time subroutine multiplies */
#include "complex.h"
#include "su3.h"
#include <stdio.h>
#include <stdlib.h>
#include <sched.h>
#include <error.h>
#include <asm/msr.h>
#include <fcntl.h>
#include <unistd.h>

void sse_mult(float,float *);
void fpu_mult(float,float *);

int main(int argc, char **argv)
{
  int i,iter=1;
  struct sched_param param={sched_priority:20};
  volatile unsigned long long timeA, timeMILC, timeSSE;
  unsigned int seed=1;
  int randomfd;
  float value = 1.000001f, out=2.0f;
  
  if ((randomfd=open("/dev/urandom", O_RDONLY)) < 0)
    perror("Attempt to open /dev/urandom");
  if (read(randomfd, &seed, sizeof(seed)) < sizeof(seed))
    perror("Attempt to read /dev/urandom");
  close(randomfd);
  srand(seed);

  if (sched_setscheduler(0, SCHED_FIFO, &param) < 0)
    perror("Attempt to put in real time queue");

  if (argc > 1) sscanf(argv[1],"%d",&iter);
  
  rdtscll(timeA);
  for (i=0; i<iter; i++) {
    fpu_mult(value,&out);
  }
  rdtscll(timeMILC);
  timeMILC -= timeA;
  printf("Output value, FPU: %f\n", out);
  out=2.0f;

  rdtscll(timeA);
  for (i=0; i<iter; i++) {
    sse_mult(value,&out);
  }
  rdtscll(timeSSE);
  timeSSE -= timeA;
  printf("Output value, SSE: %f\n", out);

  printf("Time per 100 fpu multiplies: %Lu\n", timeMILC/(unsigned long long)iter);
  printf("Time per 100 sse multiplies: %Lu\n", timeSSE/(unsigned long long)iter);

  exit(0);
}


    
